﻿using System.Security.Principal;
using Microsoft.Win32;
using net.praqma.systemlog;

namespace net.praqma.troja.library
{
    public class Configuration
    {
        private const string SUBKEY = @"SOFTWARE\Praqma\Praqma Troja Service";

        private Configuration() { }

        public static string FilterUrl
        {
            get { return getValue("filter_file_path") as string; }
        }

        public static string FilterSchemaUrl
        {
            get { return getValue("filter_schema_path") as string; }
        }

        public static string LogFile
        {
            get { return getValue("log_file_path") as string; }
        }

        public static string LogSchemaUrl
        {
            get { return getValue("log_schema_path") as string; }
        }

        private static object getValue(string name)
        {
            RegistryKey key = Registry.CurrentUser.OpenSubKey(SUBKEY);

            if (key == null)
            {
                loadSettings();
            }

            object value = Registry.CurrentUser.OpenSubKey(SUBKEY).GetValue(name) as string;

            if (value == null)
            {
                TrojaException ex = new TrojaException(string.Format("Configuration error: {0} not set", name));
                SystemLog.WriteError(WindowsIdentity.GetCurrent().Name, ex);
                throw ex;
            }

            return value;
        }

        private static void loadSettings()
        {
            RegistryKey source = Registry.LocalMachine.OpenSubKey(SUBKEY);

            if (source == null)
            {
                TrojaException ex = new TrojaException("Configuration error: Default configuration not set.");
                SystemLog.WriteError(WindowsIdentity.GetCurrent().Name, ex);
                throw ex;
            }

            RegistryKey destination = Registry.CurrentUser.CreateSubKey(SUBKEY);

            foreach (string name in source.GetValueNames())
            {
                RegistryValueKind kind = source.GetValueKind(name);
                object value = source.GetValue(name);

                destination.SetValue(name, value, kind);
            }
        }
    }
}
