﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Configuration;
using System.Security.Principal;
using System.Threading;
using System.Web;
using System.Web.Services;
using net.praqma.systemlog;
using net.praqma.troja.library;

namespace praqma.troja.service
{
    [WebService(Namespace = "http://praqma.net/troja/service/")]
    [WebServiceBinding(ConformsTo = WsiProfiles.BasicProfile1_1)]
    [ToolboxItem(false)]
    [System.Web.Script.Services.ScriptService]
    public class TrojaWS : System.Web.Services.WebService
    {
        private static IDictionary<string, IDictionary<Guid, Command>> userPool =
            new Dictionary<string, IDictionary<Guid, Command>>();
        private static Thread poolKeeper;

        #region Construction

        public TrojaWS()
        {
            if ((poolKeeper == null) || !poolKeeper.IsAlive)
            {
                poolKeeper = new Thread(new ThreadStart(this.PoolCleanup));
                poolKeeper.Priority = ThreadPriority.Lowest;
                poolKeeper.Start();
            }
        }

        #endregion

        #region WebMethods

        [WebMethod]
        public Guid CreateCommand(string command, string arguments, EnvironmentVariable[] environment, bool merge, int hours, int minutes, int seconds, string remark)
        {
            try
            {
                Command.FilterPath = ConfigurationSettings.AppSettings["filter_file_path"];
                Command.LogPath = ConfigurationSettings.AppSettings["log_file_path"];
                Command cmd = new Command(getMemberships(), command, arguments, environment, merge, new TimeSpan(hours, minutes, seconds), remark);
                UserCommands().Add(cmd.Id, cmd);

                return cmd.Id;
            }
            catch (IllegalCommandException ex)
            {
                SystemLog.WriteWarning(getUser(), ex);
                throw ex;
            }
            catch (Exception ex)
            {
                SystemLog.WriteError(getUser(), ex);
                throw new TrojaException("Could not create the command: " + ex.GetType().Name + " " + ex.Message);
            }
        }

        [WebMethod]
        public string GetCommands()
        {
            try
            {
                Command.LogPath = ConfigurationSettings.AppSettings["log_file_path"];
                return Command.Log.GetLog(UserCommands().Values);
            }
            catch (Exception ex)
            {
                SystemLog.WriteError(getUser(), ex);
                throw new TrojaException("Could not retrieve commands.");
            }
        }

        [WebMethod]
        public void StartAsync(Guid id)
        {
            Command command = UserCommands()[id];

            if (command == null)
            {
                Exception ex = new TrojaException("Unknown command: " + id);
                SystemLog.WriteWarning(getUser(), ex);
                throw ex;
            }

            try
            {
                command.Start(false);
            }
            catch (Exception ex)
            {
                SystemLog.WriteError(getUser(), ex);
                throw new TrojaException("Unable to start command.");
            }
        }

        [WebMethod]
        public string StartSync(Guid id)
        {
            Command command = UserCommands()[id];

            if (command == null)
            {
                Exception ex = new TrojaException("Unknown command: " + id);
                SystemLog.WriteWarning(getUser(), ex);
                throw ex;
            }

            try
            {
                command.Start(true);
            }
            catch (Exception ex)
            {
                SystemLog.WriteError(getUser(), ex);
                throw new TrojaException("Unable to start command.");
            }

            return command.GetLog();
        }

        [WebMethod]
        public string GetLog(Guid id)
        {
            Command command = UserCommands()[id];

            if (command == null)
            {
                Exception ex = new TrojaException("Unknown command: " + id);
                SystemLog.WriteWarning(getUser(), ex);
                throw ex;
            }

            try
            {
                return command.GetLog();
            }
            catch (Exception ex)
            {
                SystemLog.WriteError(getUser(), ex);
                throw new TrojaException("Could not retrieve log.");
            }
        }

        [WebMethod]
        public string KillCommand(Guid id)
        {
            Command command = UserCommands()[id];

            if (command == null)
            {
                Exception ex = new TrojaException("Unknown command: " + id);
                SystemLog.WriteWarning(getUser(), ex);
                throw ex;
            }

            try
            {
                command.Kill();
            }
            catch (Exception ex)
            {
                SystemLog.WriteError(getUser(), ex);
                throw new TrojaException("Could not kill command: " + id);
            }

            return command.GetLog();
        }

        [WebMethod]
        public string WhoAmI()
        {
            return getUser();
        }

        [WebMethod]
        public string ServiceUser()
        {
            return WindowsIdentity.GetCurrent().Name;
        }

        #endregion

        #region Private methods

        private IDictionary<Guid, Command> UserCommands()
        {
            string user = getUser();

            IDictionary<Guid, Command> commands;

            if (userPool.ContainsKey(user))
            {
                commands = userPool[user];
            }
            else
            {
                commands = new Dictionary<Guid, Command>();
                userPool[user] = commands;
            }

            return commands;
        }

        private string getUser()
        {
            return HttpContext.Current.Request.LogonUserIdentity.Name.ToLower();
        }

        private string[] getMemberships()
        {
            List<string> groups = new List<string>();
            groups.Add(getUser());
            foreach (IdentityReference group in HttpContext.Current.Request.LogonUserIdentity.Groups)
            {
                groups.Add(group.Translate(typeof(NTAccount)).ToString().ToLower());
            }
            return groups.ToArray();
        }

        /*
         * Cleans out dead commands once every hour.
         */ 
        private void PoolCleanup()
        {
            while (true)
            {
                foreach (KeyValuePair<string, IDictionary<Guid, Command>> user in userPool)
                {
                    foreach (KeyValuePair<Guid, Command> command in user.Value)
                    {
                        Command cmd = command.Value;

                        if ((cmd.HasExited) && (DateTime.Now.Subtract(cmd.ExitTime).Hours >= 1))
                        {
                            user.Value.Remove(command.Key);
                        }
                    }

                    if (user.Value.Count == 0)
                    {
                        userPool.Remove(user.Key);
                    }
                }

                DateTime nextHour = DateTime.Today.AddHours(DateTime.Now.Hour + 1);
                Thread.Sleep(nextHour - DateTime.Now);
            }
        }

        #endregion
    }
}
