﻿using System;
using System.Configuration;
using System.Security.Principal;
using System.Xml;
using net.praqma.systemlog;
using System.IO;
using System.Xml.Schema;
using System.Collections.Generic;

namespace praqma.troja.webgui
{
    public class GuiLog
    {
        private static readonly string XMLNS = @"http://praqma.net/troja/webgui/log/";
        private static readonly string VERSION = "1.0";
        private static readonly string PREFIX = "praq";
        private static readonly string XSD = ConfigurationSettings.AppSettings["guilog_schema_path"];
        private static readonly GuiLog instance = new GuiLog(ConfigurationSettings.AppSettings["guilog_path"]);

        private XmlNamespaceManager namespaceManager;
        private XmlDocument document;
        private string filePath;

        public static GuiLog INSTANCE
        { get { return instance; } }


        private GuiLog(string filePath)
        {
            try
            {
                this.filePath = filePath;
                Unmarshall();
            }
            catch (Exception ex)
            {
                SystemLog.WriteError(WindowsIdentity.GetCurrent().Name, ex);
                throw ex;
            }
        }

        private void Unmarshall()
        {
            if (File.Exists(filePath))
            {
                XmlReaderSettings settings = new XmlReaderSettings();
                settings.Schemas.Add(XMLNS, XSD);
                settings.ValidationEventHandler += new ValidationEventHandler(ValidationCallback);
                settings.ValidationType = ValidationType.Schema;

                XmlReader reader = XmlReader.Create(filePath, settings);
                namespaceManager = new XmlNamespaceManager(reader.NameTable);
                namespaceManager.AddNamespace(PREFIX, XMLNS);

                document = new XmlDocument();
                document.Load(reader);
                reader.Close();
            }
            else
            {
                document = createDocument();
                saveChanges();
            }
        }

        private void ValidationCallback(object sender, ValidationEventArgs args)
        {
            throw new XmlException(args.Message);
        }

        private XmlDocument createDocument()
        {
            XmlDocument doc = new XmlDocument();
            doc.AppendChild(doc.CreateXmlDeclaration("1.0", "utf-8", null));

            namespaceManager = new XmlNamespaceManager(doc.NameTable);
            namespaceManager.AddNamespace(PREFIX, XMLNS);

            XmlElement log = doc.CreateElement(PREFIX, "guilog", XMLNS);
            doc.AppendChild(log);

            XmlAttribute version = doc.CreateAttribute("version");
            version.Value = VERSION;
            log.Attributes.Append(version);

            return doc;
        }

        private void saveChanges()
        {
            XmlWriterSettings settings = new XmlWriterSettings();
            settings.CloseOutput = true;
            settings.Indent = true;
            settings.IndentChars = "\t";

            XmlWriter writer = XmlWriter.Create(filePath, settings);
            document.WriteTo(writer);
            writer.Close();
        }

        public void Add(LogItem item)
        {
            XmlElement command = document.CreateElement(PREFIX, "command", XMLNS);
            document.DocumentElement.AppendChild(command);

            XmlAttribute id = document.CreateAttribute("id");
            id.Value = item.Id.ToString();
            command.Attributes.Append(id);

            XmlElement site = document.CreateElement(PREFIX, "site", XMLNS);
            site.InnerText = item.Site;
            command.AppendChild(site);

            XmlElement datetime = document.CreateElement(PREFIX, "datetime", XMLNS);
            datetime.InnerText = item.DateTime.ToString("s");
            command.AppendChild(datetime);

            XmlElement user = document.CreateElement(PREFIX, "user", XMLNS);
            user.InnerText = item.User;
            command.AppendChild(user);

            XmlElement techUser = document.CreateElement(PREFIX, "tech_user", XMLNS);
            techUser.InnerText = item.TechUser;
            command.AppendChild(techUser);

            XmlElement view = document.CreateElement(PREFIX, "view", XMLNS);
            view.InnerText = item.View;
            command.AppendChild(view);

            XmlElement action = document.CreateElement(PREFIX, "action", XMLNS);
            action.InnerText = item.Action;
            command.AppendChild(action);

            XmlElement obj = document.CreateElement(PREFIX, "object", XMLNS);
            obj.InnerText = item.Object;
            command.AppendChild(obj);

            saveChanges();
        }

        public IList<LogItem> LogItems()
        {
            IList<LogItem> list = new List<LogItem>();

            foreach (XmlNode node in document.SelectNodes("//" + PREFIX + ":command", namespaceManager))
            {
                XmlElement command = node as XmlElement;

                Guid id = new Guid(command.GetAttribute("id"));
                string site = command.SelectSingleNode(PREFIX + ":site", namespaceManager).InnerText;
                DateTime dateTime = DateTime.Parse(command.SelectSingleNode(PREFIX + ":datetime", namespaceManager).InnerText);
                string user = command.SelectSingleNode(PREFIX + ":user", namespaceManager).InnerText;
                string techUser = command.SelectSingleNode(PREFIX + ":tech_user", namespaceManager).InnerText;
                string view = command.SelectSingleNode(PREFIX + ":view", namespaceManager).InnerText;
                string action = command.SelectSingleNode(PREFIX + ":action", namespaceManager).InnerText;
                string obj = command.SelectSingleNode(PREFIX + ":object", namespaceManager).InnerText;

                LogItem item = new LogItem(id, site, dateTime, user, techUser, view, action, obj);

                list.Add(item);
            }

            return list;
        }
    }
}
