﻿using System.Reflection;
using System.Security.Principal;
using System.ServiceModel;
using System.Text;
using System.Text.RegularExpressions;
using System.Web;
using System.Xml;
using praqma.troja.webgui.TrojaProxy;
using System;

namespace praqma.troja.webgui
{
    public class TrojaPage : System.Web.UI.Page
    {
        private static readonly string XMLNS = @"http://praqma.net/troja/log/";
        private static readonly string PREFIX = "praq";

        public string SyncCommand(string host, string arguments)
        {
            return SyncCommand(host, arguments, new TimeSpan(0, 1, 0), "");
        }

        public string SyncCommand(string host, string arguments, TimeSpan timeout, string remark)
        {
            using (((WindowsIdentity)HttpContext.Current.User.Identity).Impersonate())
            {
                TrojaWcfClient proxy = new TrojaWcfClient();
                proxy.Endpoint.Address = new EndpointAddress(endpointUrl(host));

                Guid queryId = proxy.CreateCommand(Configuration.RatlPerl, arguments, false, timeout, remark);
                string queryXml = proxy.StartSync(queryId);
                proxy.Close();

                if (ExitCode(queryXml) != 0)
                {
                    throw new ProxyException(queryXml);
                }

                return queryXml;
            }
        }

        public string ServiceUser(string host)
        {
            TrojaWcfClient proxy = new TrojaWcfClient();
            proxy.Endpoint.Address = new EndpointAddress(endpointUrl(host));

            string serviceUser = proxy.ServiceUser();
            proxy.Close();

            return serviceUser;
        }

        public string WhoAmI
        {
            get { return HttpContext.Current.User.Identity.Name.ToLower(); }
        }

        protected string StandardOut(string xml)
        {
            return InnerText("standardOut", xml);
        }

        protected string ErrorOut(string xml)
        {
            return InnerText("errorOut", xml);
        }

        protected int ExitCode(string xml)
        {
            return int.Parse(InnerText("exitCode", xml));
        }

        private string InnerText(string elementName, string xml)
        {
            XmlDocument document = new XmlDocument();
            document.LoadXml(xml);

            XmlNamespaceManager namespaceManager = new XmlNamespaceManager(document.NameTable);
            namespaceManager.AddNamespace(PREFIX, XMLNS);

            XmlElement element = document.SelectSingleNode("//" + PREFIX + ":" + elementName, namespaceManager) as XmlElement;

            return element.InnerText;
        }

        public string HtmlEncode(string text)
        {
            return Server.HtmlEncode(text).Replace("\n", "<br />");
        }

        public string Grep(string text, string pattern)
        {
            Regex regex = new Regex(pattern, RegexOptions.Compiled | RegexOptions.IgnoreCase);

            StringBuilder sb = new StringBuilder();

            foreach (string s in text.Split(new char[] { '\n' }))
            {
                if (regex.IsMatch(s))
                    sb.Append(s + "\n");
            }

            return sb.ToString();
        }

        public bool ValidateType(string text)
        {
            //return new Regex(@"^lbtype:\S+@\\\S+$", RegexOptions.Compiled).IsMatch(text);
            return true;
        }

        public bool ValidateVersion(string obj)
        {
            //TODO
            return true;
        }

        private string endpointUrl(string host)
        {
            return string.Format(Configuration.TrojaUrlPattern, host);
        }
    }
}
